-------------------------------------------------------------------------------
--|> weizPVP
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UnitLevel = UnitLevel
local GetRealmName = GetRealmName
local GetUnitName = GetUnitName
local MAX_PLAYER_LEVEL_TABLE = MAX_PLAYER_LEVEL_TABLE
local GetExpansionLevel = GetExpansionLevel
local UnitFactionGroup = UnitFactionGroup
local wipe = wipe
local select = select
local UIParent = UIParent
local C_UI_Reload = C_UI.Reload
local LibStub = LibStub

--------------------------------------b-----------------------------------------
--|> INITIALIZE
-------------------------------------------------------------------------------
--> The start of processing; entry point.
-------------------------------------------------------------------------------
function weizPVP:OnInitialize()
	--> LOAD ACE3 DB

	weizPVP.db = LibStub("AceDB-3.0"):New("weizPVPOptions")
	if not weizPVP.db.profile.Options then
		weizPVP.db.profile.Options = {}
	end
	if not weizPVP.db.global.PlayerDB then
		weizPVP.db.global.PlayerDB = {}
	end

	--> Load Database to weizPVP.Options/PlayerDB
	weizPVP:LoadDB()

	-------------------------------------------------------------------------------
	--=> Static Variables
	-------------------------------------------------------------------------------
	--> Sorting
	weizPVP.Options.Sorting = {}
	weizPVP.Options.Sorting.NearbyInactiveTimeout = 32
	weizPVP.Options.Sorting.NearbyActiveTimeout = 20

	-------------------------------------------------------------------------------
	--=> Enviromental Variables
	--> Holds infomation about the player's stats, game settings, system settings, etc.
	-------------------------------------------------------------------------------
	--> Get Player Unit Info
	weizPVP.Player = {}
	weizPVP.Player.Level = UnitLevel("player") --> player's level
	weizPVP.Player.CurrentRealm = GetRealmName() --> player's realm name
	weizPVP.Player.Name = GetUnitName("player") --> player's profile name
	weizPVP.Player.Faction = select(1, UnitFactionGroup("player")) --> Horde/Alliance
	weizPVP.MaxPlayerLevel = MAX_PLAYER_LEVEL_TABLE[GetExpansionLevel()]

	--> Get Display Info
	weizPVP.Display = {}
	weizPVP.Display.Width = GetScreenWidth()
	weizPVP.Display.Height = GetScreenHeight()
	weizPVP.Display.xFactor = (768 / weizPVP.Display.Width) * (weizPVP.Display.Width / weizPVP.Display.Height)
	weizPVP.Display.yFactor = 768 / weizPVP.Display.Height
	weizPVP.Display.UIScale = UIParent:GetScale()
	weizPVP.Display.EffectiveScale = UIParent:GetEffectiveScale()
	weizPVP.Display.CVarUiScale = GetCVar("uiScale")
	weizPVP.Display.CurrentFPS = GetFramerate()

	--> Init Static Data Tables
	weizPVP:CreateAbilityLevelDB()
	weizPVP:CreateSpecDB()

	--> Create Core Frames and Bars
	weizPVP:CreateCoreFrames()
	weizPVP:CreateBars()
	weizPVP:CreateStatusBar()

	--> Create Crosshair Frames
	if weizPVP.Options.Crosshair.Enabled then
		weizPVP.EnableCrosshair()
		weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
	end
	--> Create Stealth Alert Frame
	weizPVP:CreateStealthAlertFrame()
	weizPVP.Addon = {}
	weizPVP.Addon.Enabled = true

	--> Save Options
	weizPVP:ApplySavedOptions()
end

-------------------------------------------------------------------------------
--|> ON ENABLE
-------------------------------------------------------------------------------
function weizPVP:OnEnable()
	weizPVP.ManageBarsDisplayed()
	weizPVP.HeaderFrame:Show()
	weizPVP.CoreFrame:Show()
	weizPVP.FooterFrame:Show()

	--> Timers
	weizPVP:ScheduleRepeatingTimer("OneSecondTimer", 1) --> 1 second timer used for periodic updating.
	weizPVP:ScheduleRepeatingTimer("SaveAddonData", 10) --> Saves data every 10 seconds

	--=> Registered Events
	-------------------------------------------------------------------------------
	--> DATA COLLECTION
	weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
	weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
	weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
	weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
	weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
	weizPVP:RegisterEvent("UNIT_HEALTH", "UnitHealthEvent")

	--> ZONE CHANGED
	weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", "PlayerEnteringBattlegroundEvent")
	weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
	weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")

	--> COMBAT/PVP
	weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
	weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

	--> PLAYER UPDATES
	weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

	--> DISPLAY CHANGES
	weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
	weizPVP:RegisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
	weizPVP.Addon.Enabled = true
end

-------------------------------------------------------------------------------
--|> ON DISABLE
-------------------------------------------------------------------------------
function weizPVP:OnDisable()
	--> Hide frames
	weizPVP.HeaderFrame:Hide()
	weizPVP.CoreFrame:Hide()
	weizPVP.FooterFrame:Hide()

	weizPVP:CancelAllTimers() --> Cancel Timers

	--> Clear current player data
	weizPVP:ClearListData()

	--=> UnRegistered Events
	-------------------------------------------------------------------------------
	--> DATA COLLECTION
	weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
	weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
	weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
	weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
	weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
	weizPVP:UnregisterEvent("UNIT_HEALTH", "UnitHealthEvent")

	--> ZONE CHANGED;
	--* Leave always active so we can enable back on zone events
	-- weizPVP:UnregisterEvent("PLAYER_ENTERING_BATTLEGROUND", "PlayerEnteringBattlegroundEvent")
	-- weizPVP:UnregisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
	-- weizPVP:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")

	--> COMBAT/PVP
	weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
	weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

	--> PLAYER UPDATES
	weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

	--> DISPLAY CHANGES
	weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
	weizPVP:UnregisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
	weizPVP.Addon.Enabled = false
end

-------------------------------------------------------------------------------
--|> LOAD SAVED VARIABLE
-------------------------------------------------------------------------------
--=> Load DB
-------------------------------------------------------------------------------
function weizPVP:LoadDB()
	--> Check for Player DB
	if weizPVP.db.global.PlayerDB then
		weizPVP.PlayerDB = weizPVP.db.global.PlayerDB
	else
		weizPVP.db.global.PlayerDB = {}
		weizPVP.PlayerDB = {}
	end

	local DBMatch
	if weizPVP.db.profile.Options.DB_VERSION and weizPVP.Options.DB_VERSION then
		DBMatch = weizPVP.db.profile.Options.DB_VERSION == weizPVP.Options.DB_VERSION
	end

	--> Check for Options
	if weizPVP.db.profile.Options and DBMatch == true then
		weizPVP.Options = weizPVP.db.profile.Options
	else
		--> LOAD DEFAULT OPTIONS

		--> mismatch warning
		if DBMatch == false then
			weizPVP:PrintAddonMessage("|cffff3838WARNING:|r The database has been updated to 2.0; previous settings (all 5 of them lol) have been wiped")
			weizPVP:PrintAddonMessage("Player Data is now seperated from the other options and is now shared between all characters for your bnet account.")
		end

		--> REMOVAL OF OLDER SETTINGS DB
		if weizPVP.db.profile.Settings then
			wipe(weizPVP.db.profile.Settings)
		end
		if weizPVP.db.profile.weizPVP_DB then
			wipe(weizPVP.db.profile.weizPVP_DB)
		end

		--> Init options
		wipe(weizPVP.db.profile.Options)
		weizPVP.db.profile.Options = {}
		weizPVP.SetDefaultOptionsTable()
		weizPVP.db.profile.Options = weizPVP.Options
	end

	--> Add in new options if they don't exist in saved variables
	if not weizPVP.Options.EnabledInBattlegrounds then
		weizPVP.Options.EnabledInBattlegrounds = false
	end
	if not weizPVP.Options.AudioAlerts then
		weizPVP.Options.AudioAlerts = {}
		weizPVP.Options.AudioAlerts.DetectedPlayerSound = false
	end
	if not weizPVP.Options.Crosshair then
		weizPVP.Options.Crosshair = {}
		weizPVP.Options.Crosshair.Enabled = true
	end
	if not weizPVP.Options.Crosshair.NameEnabled then
		weizPVP.Options.Crosshair.NameEnabled = false
	end
	if not weizPVP.Options.Crosshair.GuildEnabled then
		weizPVP.Options.Crosshair.GuildEnabled = false
	end
	if not weizPVP.Options.LagMode then
		weizPVP.Options.LagMode = {}
		weizPVP.Options.LagMode.Enabled = true
	end
	if not weizPVP.Options.LagMode.FPSThreshold then
		weizPVP.Options.LagMode.FPSThreshold = 30
	end
	-- if not weizPVP.Options.LagMode.OnlyInCombat then
	-- 	weizPVP.Options.LagMode.OnlyInCombat = false
	-- end

	--> Apply current settings
	weizPVP:SaveAddonData()
end

--=> Save Addon Data
-------------------------------------------------------------------------------
function weizPVP:SaveAddonData()
	weizPVP.db.global.PlayerDB = weizPVP.PlayerDB
	weizPVP.db.profile.Options = weizPVP.Options
end

--=> Reset All
-------------------------------------------------------------------------------
function weizPVP:ResetAll()
	if weizPVP.db.profile then
		wipe(weizPVP.db.profile)
	end
	if weizPVP.db.profile then
		wipe(weizPVP.db.profile)
	end
	if weizPVP.db.global then
		wipe(weizPVP.db.global)
	end
	if weizPVP.PlayerDB then
		wipe(weizPVP.PlayerDB)
	end
	if weizPVP.Options then
		wipe(weizPVP.Options)
	end
	wipe(weizPVP.db)

	C_UI_Reload()
end
